@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2006-2007 Zope Corporation and Contributors.
+# Copyright (c) 2006-2008 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -110,7 +110,6 @@ class Recipe:
         
         options = self.options
         location = options['location']
-        
         # Don't do this if we have a manual zope.conf
         zope_conf_path = options.get('zope-conf', None)
         if zope_conf_path is not None:
@@ -155,16 +154,24 @@ class Recipe:
             
             base_dir = self.buildout['buildout']['directory']
             var_dir = options.get('var', os.path.join(base_dir, 'var'))
-            
-            event_log_name = options.get('event-log', os.path.sep.join(('log', self.name + '.log',)))
-            event_log = os.path.join(var_dir, event_log_name)
-            event_log_dir = os.path.dirname(event_log)
-            if not os.path.exists(event_log_dir):
-                os.makedirs(event_log_dir)
-            
+
             event_log_level = options.get('event-log-level', 'INFO')
             
-            z_log_name = options.get('z-log', os.path.sep.join(('log', self.name + '-Z2.log',)))
+            custom_event_log = options.get('event-log-custom', None)
+            default_log = os.path.sep.join(('log', self.name + '.log',))
+            # log file
+            if custom_event_log is None: 
+                event_log_name = options.get('event-log', default_log)
+                event_file = os.path.join(var_dir, event_log_name)
+                event_log_dir = os.path.dirname(event_file)
+                if not os.path.exists(event_log_dir):
+                    os.makedirs(event_log_dir)
+                event_log = event_logfile % {'event_logfile': event_file, 'event_log_level': event_log_level}
+            # custom log 
+            else:
+                event_log = custom_event_log
+            
+            z_log_name = options.get('z2-log', os.path.sep.join(('log', self.name + '-Z2.log',)))
             z_log = os.path.join(var_dir, z_log_name)
             z_log_dir = os.path.dirname(z_log)
             if not os.path.exists(z_log_dir):
@@ -176,19 +183,49 @@ class Recipe:
             if default_zpublisher_encoding:
                 default_zpublisher_encoding = 'default-zpublisher-encoding %s' % default_zpublisher_encoding
             
-            file_storage = options.get('file-storage', os.path.sep.join(('filestorage', 'Data.fs',)))
-            file_storage = os.path.join(var_dir, file_storage)
-            file_storage_dir = os.path.dirname(file_storage)
-            if not os.path.exists(file_storage_dir):
-                os.makedirs(file_storage_dir)
-            storage_snippet = file_storage_template % file_storage
+            relstorage = options.get('rel-storage', None)
+            if relstorage is not None:
+                rel_storage = {'dbname': 'test', 'host': 'localhost',
+                               'user': 'test', 'password': 'test'}
+                def _extract(el):
+                    if el.strip('') == '':
+                        return None
+                    el = el.split(' ')
+                    if len(el) != 2:
+                        return None
+                    return el[0].strip(), el[1].strip()
 
-            blob_storage = options.get('blob-storage', None)
-            if blob_storage:
-                blob_storage = os.path.join(base_dir, blob_storage)
-                if not os.path.exists(blob_storage):
-                    os.makedirs(blob_storage)
-                storage_snippet = blob_storage_template % (blob_storage, file_storage)
+                elements = [_extract(el) for el in relstorage.split('\n') 
+                            if _extract(el) is not None]
+                
+                type_ = 'postgresql' 
+                for key, value in elements:
+                    if key == 'type':
+                        type_ = value
+                    elif key in rel_storage:
+                        rel_storage[key] = value
+
+                if type_ == 'postgresql': 
+                    storage_snippet = rel_storage_postgres_template % rel_storage
+                else:
+                    storage_snippet = rel_storage_oracle_template % rel_storage
+
+            else:
+                file_storage = options.get('file-storage', 
+                                           os.path.sep.join(('filestorage', 
+                                                             'Data.fs',)))
+                file_storage = os.path.join(var_dir, file_storage)
+                file_storage_dir = os.path.dirname(file_storage)
+                if not os.path.exists(file_storage_dir):
+                    os.makedirs(file_storage_dir)
+                storage_snippet = file_storage_template % file_storage
+
+                blob_storage = options.get('blob-storage', None)
+                if blob_storage:
+                    blob_storage = os.path.join(base_dir, blob_storage)
+                    if not os.path.exists(blob_storage):
+                        os.makedirs(blob_storage)
+                    storage_snippet = blob_storage_template % (blob_storage, file_storage)
                 
             zserver_threads = options.get('zserver-threads', '')
             if zserver_threads:
@@ -229,9 +266,16 @@ class Recipe:
             pid_file = options.get(
                 'pid-file',
                 os.path.join(var_dir, self.name + '.pid'))
+            pid_file_dir = os.path.dirname(pid_file)
+            if not os.path.exists(pid_file_dir):
+                os.makedirs(pid_file_dir)
+
             lock_file = options.get(
                 'lock-file',
                 os.path.join(var_dir, self.name + '.lock'))
+            lock_file_dir = os.path.dirname(lock_file)
+            if not os.path.exists(lock_file_dir):
+                os.makedirs(lock_file_dir)
 
             zope_conf = template % dict(instance_home = instance_home,
                                         paths_lines = paths_lines,
@@ -438,6 +482,26 @@ file_storage_template="""
       path %s
     </filestorage>
 """
+rel_storage_postgres_template="""
+    %%import relstorage
+    <relstorage>
+        <postgresql>
+            # The dsn is optional, as are each of the parameters in the dsn.
+            dsn dbname='%(dbname)s' user='%(user)s' host='%(host)s' password='%(password)s'
+        </postgresql>
+    </relstorage>
+"""
+
+rel_storage_oracle_template="""
+    %%import relstorage
+    <relstorage>
+        <oracle>
+            user %(user)s
+            password %(pass)s
+            dsn %(dns)s
+        </oracle>
+    </relstorage>
+"""
 blob_storage_template="""
     # Blob-enabled FileStorage database
     <blobstorage>
@@ -472,6 +536,13 @@ zeo_blob_storage_template="""
     </zeoclient>
 """.strip()
 
+event_logfile = """
+  <logfile>
+    path %(event_logfile)s
+    level %(event_log_level)s
+  </logfile>
+""".strip()
+
 # The template used to build zope.conf
 zope_conf_template="""\
 instancehome %(instance_home)s
@@ -489,10 +560,7 @@ verbose-security %(verbose_security)s
 
 <eventlog>
   level %(event_log_level)s
-  <logfile>
-    path %(event_log)s
-    level info
-  </logfile>
+  %(event_log)s
 </eventlog>
 
 <logger access>
